package com.sample.xlsdt;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.builder.Message;
import org.kie.api.builder.Results;
import org.kie.api.definition.KiePackage;
import org.kie.api.definition.rule.Rule;
import org.kie.api.runtime.KieContainer;
import org.kie.api.runtime.KieSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class RuleTest {
	static final Logger LOG = LoggerFactory.getLogger(RuleTest.class);
	
	@Test
	public void testColor() {
		KieServices kieServices = KieServices.Factory.get();
        
		KieContainer kContainer = kieServices.getKieClasspathContainer();
        Results verifyResults = kContainer.verify();
        for (Message m : verifyResults.getMessages()) {
        	LOG.info("{}", m);
        }
        
	    LOG.info("Creating kieBase");
	    KieBase kieBase = kContainer.getKieBase();
        
        LOG.info("There should be rules: ");
        for ( KiePackage kp : kieBase.getKiePackages() ) {
        	for (Rule rule : kp.getRules()) {
        		LOG.info("kp " + kp + " rule " + rule.getName());
        	}
        }

	    LOG.info("Creating kieSession");
	    KieSession session = kieBase.newKieSession();
        
        LOG.info("Populating globals");
        Set<String> check = new HashSet<String>();
        session.setGlobal("controlSet", check);
        
        LOG.info("Now running data");
        
        Measurement mRed= new Measurement("color", "red");
        session.insert(mRed);
        session.fireAllRules();
        
        Measurement mGreen= new Measurement("color", "green");
        session.insert(mGreen);
        session.fireAllRules();
        
        Measurement mBlue= new Measurement("color", "blue");
        session.insert(mBlue);
        session.fireAllRules();
        
        LOG.info("Final checks");

	    assertEquals("Size of object in Working Memory is 3", 3, session.getObjects().size());
	    assertTrue("contains red", check.contains("red"));
	    assertTrue("contains green", check.contains("green"));
	    assertTrue("contains blue", check.contains("blue"));
        
	}

	@Test
	public void testTemperature() {
		KieServices kieServices = KieServices.Factory.get();

		KieContainer kContainer = kieServices.getKieClasspathContainer();
        Results verifyResults = kContainer.verify();
        for (Message m : verifyResults.getMessages()) {
        	LOG.info("{}", m);
        }

	    LOG.info("Creating kieBase");
	    KieBase kieBase = kContainer.getKieBase();

        LOG.info("There should be rules: ");
        for ( KiePackage kp : kieBase.getKiePackages() ) {
        	for (Rule rule : kp.getRules()) {
        		LOG.info("kp " + kp + " rule " + rule.getName());
        	}
        }

	    LOG.info("Creating kieSession");
	    KieSession session = kieBase.newKieSession();

        LOG.info("Populating globals");
        List<String> checkList = new ArrayList<String>();
        session.setGlobal("checkList", checkList);
        
        LOG.info("Now running data");

        Measurement t1= new Measurement("temperature", "1");
        session.insert(t1);
        session.fireAllRules();

        Measurement t2= new Measurement("temperature", "100");
        session.insert(t2);
        session.fireAllRules();

        Measurement t3= new Measurement("temperature", "-1");
        session.insert(t3);
        session.fireAllRules();

        LOG.info("Final checks");

	    assertEquals("Size of object in Working Memory is 3", 3, session.getObjects().size());
	    assertEquals("Number of detected temperatures", 4, checkList.size());
	    assertTrue("contains cold", checkList.contains("cold"));
	    assertTrue("contains warm", checkList.contains("warm"));
	    assertFalse("contains boiling", checkList.contains("boling"));

	}
}